/*
 * AM-HRR3 receiver test
 *
 * Homeeasy protocol receiver for the new protocol.
 *
 * This protocol isn't documented so well, so here goes (this is largely from memory).
 *
 * The data is encoded on the wire (aerial) as a Manchester code.
 *
 * A latch of 275us high, 2675us low is sent before the data.
 * There is a gap of 10ms between each message.
 *
 * 0 = holding the line high for 275us then low for 275us.
 * 1 = holding the line high for 275us then low for 1225us.
 *
 * The timings seem to vary quite noticably between devices.  HE300 devices have a
 * low for about 1300us for a 1 whereas HE303 devices seem to have a low of about
 * 1100us.  If this script does not detect your signals try relaxing the timing
 * conditions.
 *
 * Each actual bit of data is encoded as two bits on the wire as:
 * Data 0 = Wire 01
 * Data 1 = Wire 10
 *
 * The actual message is 32 bits of data (64 wire bits):
 * bits 0-25: the group code - a 26bit number assigned to controllers.
 * bit 26: group flag
 * bit 27: on/off flag
 * bits 28-31: the device code - a 4bit number.
 *
 * The group flag just seems to be a separate set of addresses you can program devices
 * to and doesn't trigger the dim cycle when sending two ONs.
 *
 * There's extra support for setting devices to a specific dim-level that you get
 * with the HE100 ultimate remote control. I think this involves sending a Wire 11 at
 * the on/off flag position and then the message is longer with another 4 bits for dim
 * level.
 * Need to look into this.
 *
 * Barnaby Gray 12/2008
 * Peter Mead   09/2009
 * Modified 2015-12-10: Ben Smeets: Added debug flags, simplified decoding loop and redone output printing
 *                                  removed wait for pause and do just a wait for a sync
 *                                  Added hook for decryption routine 
 *                                  Modified timings
 */
 
 /*   26 code bits: 52
       1 group       2
       1 on/of       2
       2 channel     4 or dev
       2 dev         4    dim
       */
#include "Arduino.h"
#include "skip32.h"

// show debug output
#define DEBUG
#undef DEBUG
// show timing after sync
#define DEBUG2
#undef DEBUG2


#define  REPETITIONS 1
#define  THRES ((REPETITIONS-1)/2)

#define SYNC 1
#define PAUSE 2
#define WAIT 0

int rxPin = 8;
int ledpin = 13;

unsigned long t1 = 0; // Latch 1 time only needed for debugging purposes
unsigned long t2 = 0; //latch 2 time only needed for debugging purposes.

/**
 * Here is the unique Sender/Transmitter code. Use the Serial print outs to identify your Transmitter code.
 * your transmitter code: max value is 
 *                      0x3FFFFFF if you use hex input
 *                      67108863  if you use decimal input
 */
unsigned long Sendercode = 12; 

#define ENCRYPTION
// Here we set the 128 bit key that you use. Change this key to your value.
unsigned char key[16]= {0xff, 0xff, 0xff, 0xff, \
                        0xff, 0xff, 0xff, 0xfe, \
                        0xff, 0xff, 0xff, 0xfd, \
                        0xff, 0xff, 0xff, 0xfc};
#undef ENCRYPTION

// make the skip32 crypto engine
skip32 crypt;

void setup()
{ 
  pinMode(rxPin, INPUT);
  pinMode(ledpin,OUTPUT);
  Serial.begin(9600);
 #ifdef ENCRYPTION 
  // set the key in the decryption engine
  crypt.set_key(key,16);
 #endif
}

// array were we store the recorded timings of the data in the packet
unsigned long ts[66];

/** 
 * Here starts the receiver loop
 */
void loop()
{
  int packet_cnt = 0;  // we expect 5 packets

  int i, symb;
  unsigned long t;

  byte prevBit;
  byte bit;
  unsigned long receivedData = 0;
  unsigned long sender = 0;
  bool group = false;
  bool command = false;
  unsigned int recipient = 0;
  int error;

  // we reset count of the ones and zeros in the data block
#ifdef DEBUG  
 Serial.println("start");
#endif
  while (packet_cnt < REPETITIONS) {
    receivedData = 0;
    t = 0;
    i = 0;
    error = 1;
    symb = WAIT;
    // loop until we get a SYNC
    while (symb != SYNC) {
      t = pulseIn(rxPin, LOW, 90000);
      //Serial.println(t, DEC);
      symb = symbol(t);
     //if (symb != WAIT) Serial.println(symb, DEC);
    }

    // data
    for (i = 0; i < 64; i++) {
      //delayMicroseconds(50);
      ts[i] =  pulseIn(rxPin, LOW, 3000);
    }
    i = 0;
    while (i < 64)
    {
      t = ts[i];
      
      if (t == 0) goto error;
      if (t > 150 && t < 450)
      { bit = 1;
      }
      else if (t > 1000 && t < 1800)
      { bit = 0;
      }
      else
      { //i = 0;
#ifdef DEBUG2
      Serial.print(i,DEC);
      Serial.print(" ");
      Serial.print(t,DEC);
      Serial.print("\n");      
#endif
        goto error;
      }
#ifdef DEBUG2      
      Serial.print(i,DEC);
      Serial.print(" ");
      Serial.print(t,DEC);
      Serial.print(" ");int rxpin=8;
      Serial.print(bit,DEC);
      Serial.print("\n");
#endif     
      if ((i & 1) == 1)
      {
        if ((prevBit ^ bit) == 0)
        { // must be either 01 or 10, cannot be 00 or 11
#ifdef DEBUG 
          Serial.print("00 or 11 \n");
#endif
          goto error;
        }
        receivedData <<= 1;
        receivedData |= prevBit;
      }
      prevBit = bit;
      ++i;
    }
    packet_cnt++;

  }
  error = 0;
error:;
 if (error==0) {
 #ifdef ENCRYPTION
    // decrypt
 	decrypt(&receivedData);
 #endif
    sender = 0x3FFFFFF & (receivedData >> 6);
    command = 1^ ((receivedData >> 4) & 0x01);
    group   = 1^ ((receivedData >> 5) & 0x01);
    recipient = (unsigned int)receivedData & 0xF;

#ifdef DEBUG
    Serial.print("cmd:");PrintBinLong(receivedData);Serial.print("\n");
#endif

    if (Sendercode==0) 
    	printResult(sender, recipient, command, group);
    	
    if (sender == Sendercode) {
        Serial.print("Will turn ");
        command ? Serial.println("on") : Serial.println("off");
    }
 }
#ifdef DEBUG
 else
   Serial.println("error");
#endif
}

/**
 * decrypt data recovered from channel 
 * this should be 32 binary symbols
 * we decrypt them and put the decrypted symbols back
 */
void decrypt(unsigned long *data){

  crypt.decrypt32(data);
}

/**
 * decode what symbol was sent
 */
static int symbol(unsigned long t) {
  if (t == 0) return WAIT;
  if ((t > 9000) && (t < 11000))
    return PAUSE; // pause
  if ((t > 2000) && (t < 3500))
    return SYNC; // sync
  return 0;
}

/**
 * Print the details of the message.
 */
static void printResult(unsigned long sender, unsigned int recipient, bool on, bool group)
{
        Serial.print("message");
	//PrintBinLong(sender);
        Serial.print("\n");
	Serial.print("sender    ");
	Serial.println(sender);
	
	Serial.print("recipient ");
	Serial.println(recipient);
	
	Serial.print("on        ");
	Serial.println(on);
	if (on==1) 
		digitalWrite(ledpin,HIGH);
	else
		digitalWrite(ledpin,LOW);
	
	Serial.print("group     ");
	Serial.println(group);
	
	Serial.println();
}

static void PrintBinLong(unsigned long data){
  int i;
  unsigned long bitt;
  for (i=0;i<32;i++){
    bitt = 0x1 & (data >> (31-i));
    Serial.print(bitt,BIN);
  }
}
